<?php
/**
 * @desc Implements CRUD operations on various DB tables by means of one controller.
 * 
 * @author Vladimir Kovarsky <vkovarsky@gmail.com>
 * 
 * @copyright Copyright &copy; 2009 Algo-rithm
 * 
 * @version 1.0 
 */

class CrudController extends CExtController
{
	const PAGE_SIZE=10;

	/**
	 * @var string specifies the default action to be 'list'.
	 */
	public $defaultAction='list';

	/**
	 * @var CActiveRecord the currently loaded data model instance.
	 */
	private $_item;

    /**
    * @var string define layout here
    */
    public $layout = 'main';
    
    /**
    * @desc system properties - don't customize!
    */
    public  $modelName;
    public  $modelItem;
    private $pluralName;
    private $dataForSelect;
    
    public function init()
    {
        
        if (isset($_GET['model'])) {
            $this->modelName = ucfirst($_GET['model']);
            $this->modelItem = $model = new $this->modelName;  
            Yii::app()->user->setState('modelName', $this->modelName);
            
            if (isset($model->pluralName)) 
                $pluralName = Yii::t('yii',$model->pluralName,/*params=*/array(), 'crudMessages') ;
            else
                $pluralName = Yii::t('yii',$this->modelName,/*params=*/array(), 'crudMessages') ;
            
            $this->pluralName = $pluralName;
        }
    }

    /**
     * @return array action filters
     */
    public function filters()
    {
        return array(
            'accessControl'
        );
    }

	/**
	 * Specifies the access control rules.
	 * This method is used by the 'accessControl' filter.
	 * @return array access control rules
	 */
	public function accessRules()
	{
		return array(
			array('allow',  // allow all authorized users to perform all actions
				'users'=>array('@'),
			),

			array('deny',  // deny all users (but authorized!)
				'users'=>array('*'),
			),
		);
	}

	/**
	 * Creates a new item.
	 * If creation is successful, the browser will be redirected to the previous page.
	 */
	public function actionCreate()
	{
        $action    = 'create';
        $actionTitle = Yii::t('yii','New record',/*params=*/array(), 'crudMessages') . ' - ' . $this->pluralName;
        $modelName = $this->modelName;
        $newItem   = $this->modelItem;
        
		if(isset($_POST[$modelName])) {
			$newItem->attributes = $_POST[$modelName];
			if ($newItem->save())
				if (Yii::app()->user->getState('redirectURI')!=null)
                    $this->redirect(Yii::app()->user->getState('redirectURI'));
                else
                    $this->redirect(array('list', 'model'=>strtolower($modelName)));
		} elseif (isset($_SERVER['HTTP_REFERER'])) {
            Yii::app()->user->setState('redirectURI', $_SERVER['HTTP_REFERER']); 
        }
        
        //data for DropDown lists
        $this->getDataForSelect($newItem);     
        
        $this->render('createUpdate',array(
                                'action'=> $action,
                                'actionTitle'=>$actionTitle,
                                'item'  => $newItem, 
                                )
        );
	}

    /**
    * Returns at the previous page if "Cancel"
    * 
    */
    public function actionCancel()
    {
        if (Yii::app()->user->getState('redirectURI')!=null)
            $this->redirect(Yii::app()->user->getState('redirectURI'));
        else
            $this->redirect(array('list', 'model'=>strtolower(Yii::app()->user->getState('modelName'))));
    }
    
	/**
	 * Updates a particular customer.
	 * If update is successful, the browser will be redirected to the 'show' page.
	 */
	public function actionUpdate()
	{
		$action      = 'update';
        $actionTitle = Yii::t('yii','Updating record #{id}',array('{id}'=>$_GET['id']), 'crudMessages') . ' - ' . $this->pluralName;
        $item        = $this->loadItem();
        $item->with(array_keys($item->relations()));

        $modelName = $this->modelName;

		if(isset($_POST[$modelName])) {
			$item->attributes = $_POST[$modelName];
			if($item->save())
                if (Yii::app()->user->getState('redirectURI')!=null)
                    $this->redirect(Yii::app()->user->getState('redirectURI'));
                else
                    $this->redirect(array('list','model'=>strtolower($modelName)));
        } elseif (isset($_SERVER['HTTP_REFERER'])) {
            //remeber previous page - if enter first time (button "Save" was not pressed)
            Yii::app()->user->setState('redirectURI', $_SERVER['HTTP_REFERER']);
		}
		
        //data for DropDown lists
        $this->getDataForSelect($item);     

        $this->render('createUpdate',array(
                        'action'     => $action,
                        'actionTitle'=>$actionTitle,
                        'item'       => $item,
                        )
        );

	}

	/**
	 * Deletes a particular customer.
	 * If deletion is successful, the browser will be redirected to the 'list' page.
	 */
	public function actionDelete()
	{
		if(Yii::app()->request->isPostRequest)
		{       
            $this->loadItem($_POST['recordId'])->delete();
			$this->redirect($this->createUrl('list', array('model'=>strtolower($this->modelName))));
		}
		else
			throw new CHttpException(500, Yii::t('yii','Invalid request. Please do not repeat this request again.'));
	}

	/**
	 * Manages all items.
	 */
	public function actionList()
	{
        if (isset($this->modelItem->pluralName))
            $pluralName = $this->pluralName;
        else
            $pluralName = $this->pluralName . ' list';
        
        $actionTitle = Yii::t('yii',$pluralName,/*params=*/array(), 'crudMessages');
        $modelName   = $this->modelName;
        $item        = $this->modelItem;
        
		$criteria=new CDbCriteria;
        
        //pagination
		$pages=new CPagination($item->model()->count());
		$pages->pageSize=self::PAGE_SIZE;
		$pages->applyLimit($criteria);

		//sort
        $sort=new CSort($modelName);
        
        $sortAttributes = array();
        $columnAttributes = array();
        $foreignList    = array();
        $columns   = $item->tableSchema->columns;
        $relations = $item->relations();
        $labels    = $item->attributeLabels();

 
        foreach ($relations as $key=>$val) {
            $foreignList[$val[2]] = $key;
        }
        
        $sortAttributes[$item->tableName().'.id'] = 'id'; 
        $columnAttributes['id'] = '';       
        unset($columns['id']);
        
        foreach ($columns as $key=>$val) {
            if ($val->isForeignKey) {
                $sortAttributes[$foreignList[$key].'.name'] = $foreignList[$key];
                $columnAttributes[$foreignList[$key]] = 'name';
            } else {
                $sortAttributes[$item->tableName().'.'.$key] = $key;
                $columnAttributes[$key] = '';
            }
        }
        
        $sort->attributes = $sortAttributes;
		$sort->applyOrder($criteria);

		$list=$item->model()->with(array_keys($relations))->findAll($criteria);
        
		$this->render('list',array(
            'actionTitle'=>$actionTitle,
            'sortAttributes'=>$sortAttributes,
            'columnAttributes'=>$columnAttributes,
			'list'=>$list,
			'pages'=>$pages,
			'sort'=>$sort,
		));
	}


	/**
	 * Returns the data model based on the primary key given in the GET variable.
	 * If the data model is not found, an HTTP exception will be raised.
	 * @param integer the primary key value. Defaults to null, meaning using the 'id' GET variable
	 */
	public function loadItem($id=null)
	{
		if($this->_item===null)
		{
			if($id!==null || isset($_GET['id']))
				$this->_item=$this->modelItem->model()->findbyPk($id!==null ? $id : $_GET['id']);
			if($this->_item===null)
				throw new CHttpException(500,Yii::t('yii','The requested record does not exist.'));
		}
		return $this->_item;
	}

    /**
     * Get data for "select options"
     * @param CActveRecord  
     * @return array 
     */
    protected function getDataForSelect($model)
    {
        $relations = $model->relations();
        $data = array();
        foreach ($relations as $name=>$valueArray) {
            $modelName = new $valueArray[1];
            $data[$valueArray[2]] = CHtml::listData($modelName->model()->findAll(
                        array('select'=>'id, name')),
                        'id',
                        'name'
                        );
        }
        $this->dataForSelect = $data;
    }
    
    protected function generateInputField($model, $column, $action)
    {                                                                
        $cName = $column->name;
        if($column->type==='boolean')
            return CHtml::activeCheckBox($model, $column->name);
        else if(stripos($column->dbType,'text')!==false)
            return CHtml::activeTextArea($model, $column->name, array('rows'=>6, 'cols'=>50));
        else if($column->isForeignKey) 
        {
            return CHtml::activeDropDownList($model, $column->name, $this->dataForSelect[$column->name], ($action=='create')?array('prompt'=>' - - - - - - - - - - '):'');
        }
        else
        {
            if(preg_match('/^(password|pass|passwd|passcode)$/i',$column->name))
                $inputField='activePasswordField';
            else
                $inputField='activeTextField';

            if($column->type!=='string' || $column->size===null)
                return CHtml::$inputField($model, $cName);
            else
            {
                if(($size=$maxLength=$column->size)>60)
                    $size=60;
                return CHtml::$inputField($model, $cName, array('size'=>$size,'maxlength'=>$maxLength));
            }
        }
    }
    
    
}
